/**
 * The Su Doku puzzle board. 
 */
public class Board
{
    public static final int TYPE_NORMAL = 0;
    public static final int TYPE_INITIAL = 1;
    
    /** The currently set numbers, [x][y] (1-9, 0 == not set yet). */
    public int[][] numbers = new int[ 9 ][ 9 ];

    public int[][] types = new int[ 9 ][ 9 ];
    
    public Board()
    {
    }
    
    public Board( Board b )
    {
        if ( b != null )
            copyFrom( b );
    }
    
    /**
     * Generates a new Su Doku puzzle.
     * 
     * @param diff The difficulty level (0-2).
     */
    public void generate( int diff )
    {
        boolean valid = false;
        while ( !valid )
        {
	        int remove_amt = 35 + diff * 9;
	        
	        // Generate a new board (full solution)
	        System.out.println( "Solving a clear board" );
	        Board b = new Board();
	        b.solve();
	        copyFrom( b );
	        
	        valid = true;
	        
	        // Remove numbers until we hit our target
	        System.out.println( "Removing numbers" );
	        while ( valid && remove_amt-- > 0 )
	        {
	            // Pick a potential place to remove
	            int tries = 100;
	            while ( tries-- > 0 )
	            {
	            	int x = Res.randomInt( 8 );
	            	int y = Res.randomInt( 8 );
	            	
	            	if ( numbers[ x ][ y ] <= 0 )
	            	    continue;
	            	
	            	b.copyFrom( this );
	            	b.numbers[ x ][ y ] = 0;
	            	b.solveDeterministic();
	            	
	            	if ( b.isComplete() )
	            	{ // Found one!
	            	    numbers[ x ][ y ] = 0;
	            	    break;
	            	}
	            }
	            
	            if ( tries <= 0 )
	                valid = false;
	        }
	        
	        if ( !valid )
	            System.out.println( "FAILED! Had " + (remove_amt+1) + " numbers left to remove!" );
        }
        
        // Mark as "initial"
        for ( int y = 0; y < 9; ++y )
        {
            for ( int x = 0; x < 9; ++x )
            {
                if ( numbers[ x ][ y ] > 0 )
                    types[ x ][ y ] = TYPE_INITIAL;
            }
        }
        
        System.out.println( "Generated!" );
    }
    
    /**
     * Copies the board from the other board.
     * 
     * @param b The source board.
     */
    public void copyFrom( Board b )
    {
        for ( int y = 0; y < 9; ++y )
        {
            for ( int x = 0; x < 9; ++x )
            {
                numbers[ x ][ y ] = b.numbers[ x ][ y ];
                types[ x ][ y ] = b.types[ x ][ y ];
            }
        }
    }
    
    /**
     * Tests if the given coordinate position is valid in the current puzzle.
     * 
     * @param x The x coordinate.
     * @param y The y coordinate.
     * @return True if valid, false if not.
     */
    public boolean isValid( int x, int y )
    {
        int num = numbers[ x ][ y ];
        if ( num <= 0 )
            return true;
        
        // Check the row & column
        for ( int i = 0; i < 9; ++i )
        {
            int r = numbers[ i ][ y ];
            int c = numbers[ x ][ i ];
            
            if ( i != x && r == num )
                return false;
            if ( i != y && c == num )
                return false;
        }
        
        // Check the 3x3 square
        int sx = (x/3)*3;
        int sy = (y/3)*3;
        for ( int iy = 0; iy < 3; ++iy )
        {
            int ty = sy + iy;
            
            for ( int ix = 0; ix < 3; ++ix )
            {
                int tx = sx + ix;
                
                if ( tx != x || ty != y )
                {
                    if ( numbers[ tx ][ ty ] == num )
                        return false;
                }
            }
        }
        
        return true;
    }
    
    /**
     * Tests if the board is complete.
     * 
     * @return True if done, false otherwise.
     */
    public boolean isComplete()
    {
        for ( int y = 0; y < 9; ++y )
        {
            for ( int x = 0; x < 9; ++x )
            {
                if ( numbers[ x ][ y ] <= 0 )
                    return false;
                if ( !isValid( x, y ) )
                    return false;
            }
        }

        return true;
    }
    
    /**
     * Solves this Su Doku puzzle.
     */
    public boolean solve()
    {
        int[] possibilities = new int[ 9 ];
        
        boolean solved_any = false;
        do
        {
            solved_any = false;
            
            solveDeterministic();
            
            if ( isComplete() )
                return true;

            // Hard backtracing algo
	        for ( int y = 0; y < 9; ++y )
	        {
	            for ( int x = 0; x < 9; ++x )
	            {
	                int found = getPossibilities( x, y, possibilities );
	                
	                if ( found <= 0 )
	                    continue;
	                
	                // Permutate the possibilities
	                for ( int i = 0; i < 9; ++i )
	                {
	                    int i1 = Res.randomInt( found - 1 ); 
	                    int i2 = Res.randomInt( found - 1 );
	                    
	                    int t = possibilities[ i1 ];
	                    possibilities[ i1 ] = possibilities[ i2 ];
	                    possibilities[ i2 ] = t;
	                }
	                
	                // Try until we find a good match
	                boolean ok = false;
	                for ( int i = 0; i < found; ++i )
	                {
	                    numbers[ x ][ y ] = possibilities[ i ];
	                    if ( solve() )
	                    {
	                        solved_any = true;
	                        ok = true;
	                        break;
	                    }
	                }
	                
	                // None matched?!?
	                if ( !ok )
	                {
	                    numbers[ x ][ y ] = 0;
	                    return false;
	                }
	                break;
	            }
	            
	            if ( solved_any )
	                break;
	        }
        } while ( solved_any );
        
        return isComplete();
    }
    
    /**
     * Solves the deterministicly placed values as long as possible.
     */
    private void solveDeterministic()
    {
        boolean solved_any = false;
        do
        {
            solved_any = false;
            
            // Try all "fittings" and fill those which simply cannot have any but one solution
	        for ( int y = 0; y < 9; ++y )
	        {
	            for ( int x = 0; x < 9; ++x )
	            {
	                int found = getPossibilities( x, y, g_temp_arr );
	                
	                // If only 1 value to place, then place it
	                if ( found == 1 )
	                {
	                    numbers[ x ][ y ] = g_temp_arr[ 0 ];
	                    solved_any = true;
	                }
	            }
	        }

            // Try all rows
            for ( int r = 0; r < 9; ++r )
            {
                for ( int v = 1; v <= 9; ++v )
                {
                    int pos = -1;
                    int found_amt = 0;
                    for ( int x = 0; x < 9; ++x )
                    {
                        if ( numbers[ x ][ r ] > 0 )
                            continue;
                        numbers[ x ][ r ] = v;
                        if ( isValid( x, r ) )
                        {
                            pos = x;
                            ++found_amt;
                        }
                        numbers[ x ][ r ] = 0;
                    }
                    
                    if ( found_amt == 1 )
                    {
                        numbers[ pos ][ r ] = v;
	                    solved_any = true;
                    }
                }
            }

            // Try all columns
            for ( int c = 0; c < 9; ++c )
            {
                for ( int v = 1; v <= 9; ++v )
                {
                    int pos = -1;
                    int found_amt = 0;
                    for ( int y = 0; y < 9; ++y )
                    {
                        if ( numbers[ c ][ y ] > 0 )
                            continue;
                        numbers[ c ][ y ] = v;
                        if ( isValid( c, y ) )
                        {
                            pos = y;
                            ++found_amt;
                        }
                        numbers[ c ][ y ] = 0;
                    }
                    
                    if ( found_amt == 1 )
                    {
                        numbers[ c ][ pos ] = v;
	                    solved_any = true;
	                }
                }
            }
            
            // Try all 3x3 blocks
            for ( int by = 0; by < 9; by += 3 )
            {
                for ( int bx = 0; bx < 9; bx += 3 )
                {
                    for ( int v = 1; v <= 9; ++v )
                    {
                        int pos_x = -1;
                        int pos_y = -1;
                        int found_amt = 0;
                    
                        for ( int y = by; y < (by+3); ++y )
                        {
                            for ( int x = bx; x < (bx+3); ++x )
                            {
                                if ( numbers[ x ][ y ] > 0 )
                                    continue;
                                numbers[ x ][ y ] = v;
                                if ( isValid( x, y ) )
                                {
                                    pos_x = x;
                                    pos_y = y;
                                    ++found_amt;
                                }
                                numbers[ x ][ y ] = 0;
                            }
                        }
                        
                        if ( found_amt == 1 )
                        {
                            numbers[ pos_x ][ pos_y ] = v;
    	                    solved_any = true;
    	                }
                    }
                }
            }
        } while ( solved_any );
    }
    
    /**
     * Returns the number of possibilities for a given square.
     * 
     * @param x The x index.
     * @param y The y index.
     * @param dest_arr The destination array (should have at least 9 elements, can be null). 
     * @return The number of different numbers that can be placed at the given position.
     */
    private int getPossibilities( int x, int y, int[] dest_arr )
    {
        if ( numbers[ x ][ y ] > 0 )
            return 0;
        
        int found = 0;
        for ( int v = 1; v <= 9; ++v )
        {
            numbers[ x ][ y ] = v;
            if ( isValid( x, y ) )
            {
                if ( dest_arr != null )
                    dest_arr[ found ] = v;
                ++found;
            }
        }

        numbers[ x ][ y ] = 0;
        return found;
    }
    
    private static int[] g_temp_arr = new int[ 9 ];
}
